const { spawn } = require('child_process')
const { exec } = require('../db/database');

function getHigherVulnerability(cves) {
  let severitiesArray = cves.map(cve => {
    return cve.cvssV3 ? cve.cvssV3.severity : (cve.cvss2 ? cve.cvss2.severity : 'N/A')
  })

  if (severitiesArray.includes('CRITICAL'))
    return 'CRITICAL'
  if (severitiesArray.includes('HIGH'))
    return 'HIGH'
  if (severitiesArray.includes('MEDIUM'))
    return 'MEDIUM'
  if (severitiesArray.includes('LOW'))
    return 'LOW'
  if (severitiesArray.includes('NONE'))
    return 'NONE'
  return 'N/A'
}

searchSploit('Nginx', function (response) {
  console.log(response);
})

async function searchSploit(term, cb) {
  const child = spawn('searchsploit', ['-j', '--id', `${term}`])

  let fullData = null
  child.stdout.on('data', (data) => {
    fullData += data.toString()
  })

  child.stderr.on('error', (stderr) => console.error('Standard Error: ', stderr))

  child.on('error', (error) => console.error(' Error: ', data))

  child.stdout.on('end', async () => {

    fullData = fullData.replaceAll('null', '');
    fullData = JSON.parse(fullData)?.RESULTS_EXPLOIT;

    let data = []
    let exploits = await getExploitsDetailsFromMongo(fullData);

    if (!exploits.length) {
      cb({
        message: 'No exploits available',
        data: []
      })
      return;
    }

    for (let exploitIndex = 0; exploitIndex < exploits.length; exploitIndex++) {
      let cves = exploits[exploitIndex].cves;
      //Pesquisar na BD Local
      let cves_data = await getCveDetailsFromMongo(cves)
      data = [
        ...data,
        {
          ed_id: exploits[exploitIndex].exploit_id,
          severity: getHigherVulnerability(cves_data),
          title: exploits[exploitIndex].description,
          platform: exploits[exploitIndex].platform,
          type: exploits[exploitIndex].type,
          cves: cves_data
        }
      ]
    }
    cb({
      message: 'Exploits available',
      data
    })

  })

  child.on('exit', (code, signal) => {
    if (code) console.log(`Process exit with code ${code}`);
    if (signal) console.log(`Process killed with signal ${signal}`);
  })
}

async function getCveDetailsFromMongo(cves) {
  try {
    const conditions = cves.map(cve => cve.cve_id)
    const result = await exec(async (conn) => {
      return await conn
        .db(process.env.MONGODB_NAME)
        .collection("cves")
        .aggregate([
          {
            $match: {
              cve_id: {
                $in: conditions
              }
            }
          },
          { $project: { _id: 0 } }
        ]).toArray();
    });
    return result
  } catch (error) {
    return error
  }
}

async function getExploitsDetailsFromMongo(exploits) {
  try {
    const conditions = exploits.map(exploit => exploit['EDB-ID'])
    const result = await exec(async (conn) => {
      return await conn
        .db(process.env.MONGODB_NAME)
        .collection("exploits")
        .aggregate([
          {
            $match: {
              exploit_id: {
                $in: conditions
              }
            }
          },
          { $project: { _id: 0 } }
        ]).toArray();
    });
    return result
  } catch (error) {
    return error
  }
}

module.exports = {
  searchSploit
}
